Trait sp_std::marker::Sync1.0.0[−][src]

pub unsafe auto trait Sync { }

Types for which it is safe to share references between threads.

This trait is automatically implemented when the compiler determines it’s appropriate.

The precise definition is: a type T is Sync if and only if &T is Send. In other words, if there is no possibility of undefined behavior (including data races) when passing &T references between threads.

As one would expect, primitive types like u8 and f64 are all Sync, and so are simple aggregate types containing them, like tuples, structs and enums. More examples of basic Sync types include “immutable” types like &T, and those with simple inherited mutability, such as Box<T>, Vec<T> and most other collection types. (Generic parameters need to be Sync for their container to be Sync.)

A somewhat surprising consequence of the definition is that &mut T is Sync (if T is Sync) even though it seems like that might provide unsynchronized mutation. The trick is that a mutable reference behind a shared reference (that is, & &mut T) becomes read-only, as if it were a & &T. Hence there is no risk of a data race.

Types that are not Sync are those that have “interior mutability” in a non-thread-safe form, such as Cell and RefCell. These types allow for mutation of their contents even through an immutable, shared reference. For example the set method on Cell<T> takes &self, so it requires only a shared reference &Cell<T>. The method performs no synchronization, thus Cell cannot be Sync.

Another example of a non-Sync type is the reference-counting pointer Rc. Given any reference &Rc<T>, you can clone a new Rc<T>, modifying the reference counts in a non-atomic way.

For cases when one does need thread-safe interior mutability, Rust provides atomic data types, as well as explicit locking via sync::Mutex and sync::RwLock. These types ensure that any mutation cannot cause data races, hence the types are Sync. Likewise, sync::Arc provides a thread-safe analogue of Rc.

Any types with interior mutability must also use the cell::UnsafeCell wrapper around the value(s) which can be mutated through a shared reference. Failing to doing this is undefined behavior. For example, transmute-ing from &T to &mut T is invalid.

See the Nomicon for more details about Sync.

Implementations on Foreign Types

impl Sync for Argument

impl Sync for FormatSpec

impl Sync for Alignment

impl Sync for Count

impl<T> Sync for Key<T>[src]

impl<'a> Sync for IoSliceMut<'a>[src]

impl<'a> Sync for IoSlice<'a>[src]

impl<T> Sync for SyncOnceCell<T> where
    T: Sync + Send, 
[src]

impl !Sync for ArgsOs[src]

impl !Sync for Args[src]

impl<T> Sync for JoinHandle<T>[src]

impl<T, F> Sync for SyncLazy<T, F> where
    F: Send,
    SyncOnceCell<T>: Sync, 
[src]

impl<T> !Sync for *mut T where
    T: ?Sized, 
[src]

impl<T> !Sync for *const T where
    T: ?Sized, 
[src]

impl Sync for Waker[src]

impl<'_, T> Sync for Iter<'_, T> where
    T: Sync, 
[src]

impl<'_, T> Sync for Cursor<'_, T> where
    T: Sync, 
[src]

impl<'_> Sync for Drain<'_>[src]

impl<'_, T> Sync for IterMut<'_, T> where
    T: Sync, 
[src]

impl<'_, T> Sync for CursorMut<'_, T> where
    T: Sync, 
[src]

impl<T> Sync for LinkedList<T> where
    T: Sync, 
[src]

impl Sync for isize

impl<T> Sync for [T] where
    T: Sync, 

impl Sync for [u8]

impl Sync for char

impl Sync for u128

impl Sync for u16

impl Sync for i128

impl Sync for i16

impl Sync for str

impl Sync for f64

impl Sync for u64

impl Sync for u8

impl Sync for i64

impl Sync for i8

impl<T, const N: usize> Sync for [T; N] where
    T: Sync, 

impl Sync for bool

impl Sync for f32

impl Sync for u32

impl Sync for usize

impl Sync for i32

Loading content...

Implementors

impl Sync for AtomicBool[src]

impl Sync for AtomicI81.34.0[src]

impl Sync for AtomicI161.34.0[src]

impl Sync for AtomicI321.34.0[src]

impl Sync for AtomicI641.34.0[src]

impl Sync for AtomicIsize[src]

impl Sync for AtomicU81.34.0[src]

impl Sync for AtomicU161.34.0[src]

impl Sync for AtomicU321.34.0[src]

impl Sync for AtomicU641.34.0[src]

impl Sync for AtomicUsize[src]

impl Sync for sp_std::sync::Once[src]

impl<'_, T> Sync for sp_std::collections::vec_deque::Drain<'_, T> where
    T: Sync, 
1.6.0[src]

impl<'_, T> Sync for sp_std::collections::vec_deque::IterMut<'_, T> where
    T: Sync, 
[src]

impl<'_, T> Sync for sp_std::slice::Iter<'_, T> where
    T: Sync, 
[src]

impl<'_, T> Sync for sp_std::slice::IterMut<'_, T> where
    T: Sync, 
[src]

impl<'_, T> Sync for MutexGuard<'_, T> where
    T: Sync + ?Sized, 
1.19.0[src]

impl<'_, T> Sync for RwLockReadGuard<'_, T> where
    T: Sync + ?Sized, 
1.23.0[src]

impl<'_, T> Sync for RwLockWriteGuard<'_, T> where
    T: Sync + ?Sized, 
1.23.0[src]

impl<'_, T, A> Sync for sp_std::vec::Drain<'_, T, A> where
    T: Sync,
    A: Sync + Allocator, 
1.6.0[src]

impl<Dyn> Sync for DynMetadata<Dyn> where
    Dyn: ?Sized, 
[src]

impl<T> !Sync for Cell<T> where
    T: ?Sized, 
[src]

impl<T> !Sync for RefCell<T> where
    T: ?Sized, 
[src]

impl<T> !Sync for UnsafeCell<T> where
    T: ?Sized, 
[src]

impl<T> !Sync for NonNull<T> where
    T: ?Sized, 
1.25.0[src]

NonNull pointers are not Sync because the data they reference may be aliased.

impl<T> !Sync for Rc<T> where
    T: ?Sized, 
[src]

impl<T> !Sync for sp_std::rc::Weak<T> where
    T: ?Sized, 
1.4.0[src]

impl<T> !Sync for Receiver<T>[src]

impl<T> !Sync for Sender<T>[src]

impl<T> Sync for Empty<T>1.42.0[src]

impl<T> Sync for AtomicPtr<T>[src]

impl<T> Sync for Arc<T> where
    T: Sync + Send + ?Sized, 
[src]

impl<T> Sync for Mutex<T> where
    T: Send + ?Sized, 
[src]

impl<T> Sync for RwLock<T> where
    T: Send + Sync + ?Sized, 
[src]

impl<T> Sync for sp_std::sync::Weak<T> where
    T: Sync + Send + ?Sized, 
1.4.0[src]

impl<T, A> Sync for sp_std::vec::IntoIter<T, A> where
    T: Sync,
    A: Allocator, 
[src]

Loading content...

Auto implementors

impl !Sync for OnceState

impl Sync for sp_std::cmp::Ordering

impl Sync for Infallible

impl Sync for sp_std::fmt::Alignment

impl Sync for FpCategory

impl Sync for IntErrorKind

impl Sync for SearchStep

impl Sync for sp_std::sync::atomic::Ordering

impl Sync for RecvTimeoutError

impl Sync for TryRecvError

impl Sync for AllocError

impl Sync for Global

impl Sync for Layout

impl Sync for LayoutError

impl Sync for System

impl Sync for TypeId

impl Sync for BorrowError

impl Sync for BorrowMutError

impl Sync for Error

impl Sync for SipHasher

impl Sync for NonZeroI8

impl Sync for NonZeroI16

impl Sync for NonZeroI32

impl Sync for NonZeroI64

impl Sync for NonZeroI128

impl Sync for NonZeroIsize

impl Sync for NonZeroU8

impl Sync for NonZeroU16

impl Sync for NonZeroU32

impl Sync for NonZeroU64

impl Sync for NonZeroU128

impl Sync for NonZeroUsize

impl Sync for ParseFloatError

impl Sync for ParseIntError

impl Sync for TryFromIntError

impl Sync for RangeFull

impl Sync for ParseBoolError

impl Sync for Utf8Error

impl Sync for Writer

impl Sync for RecvError

impl Sync for Barrier

impl Sync for BarrierWaitResult

impl Sync for Condvar

impl Sync for WaitTimeoutResult

impl Sync for PhantomPinned

impl<'a> !Sync for Arguments<'a>

impl<'a> !Sync for Formatter<'a>

impl<'a> Sync for CharSearcher<'a>

impl<'a> Sync for Bytes<'a>

impl<'a> Sync for CharIndices<'a>

impl<'a> Sync for Chars<'a>

impl<'a> Sync for EncodeUtf16<'a>

impl<'a> Sync for EscapeDebug<'a>

impl<'a> Sync for EscapeDefault<'a>

impl<'a> Sync for EscapeUnicode<'a>

impl<'a> Sync for Lines<'a>

impl<'a> Sync for LinesAny<'a>

impl<'a> Sync for SplitAsciiWhitespace<'a>

impl<'a> Sync for SplitWhitespace<'a>

impl<'a, 'b> !Sync for DebugList<'a, 'b>

impl<'a, 'b> !Sync for DebugMap<'a, 'b>

impl<'a, 'b> !Sync for DebugSet<'a, 'b>

impl<'a, 'b> !Sync for DebugStruct<'a, 'b>

impl<'a, 'b> !Sync for DebugTuple<'a, 'b>

impl<'a, 'b> Sync for CharSliceSearcher<'a, 'b>

impl<'a, 'b> Sync for StrSearcher<'a, 'b>

impl<'a, B: ?Sized> Sync for Cow<'a, B> where
    B: Sync,
    <B as ToOwned>::Owned: Sync, 

impl<'a, F> Sync for CharPredicateSearcher<'a, F> where
    F: Sync, 

impl<'a, I, A> Sync for Splice<'a, I, A> where
    A: Sync,
    I: Sync,
    <I as Iterator>::Item: Sync, 

impl<'a, K, V> Sync for Entry<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for sp_std::collections::btree_map::Iter<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for sp_std::collections::btree_map::IterMut<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for Keys<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for OccupiedEntry<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for OccupiedError<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for sp_std::collections::btree_map::Range<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for RangeMut<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for VacantEntry<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for Values<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V> Sync for ValuesMut<'a, K, V> where
    K: Sync,
    V: Sync, 

impl<'a, K, V, F> Sync for sp_std::collections::btree_map::DrainFilter<'a, K, V, F> where
    F: Sync,
    K: Sync,
    V: Sync, 

impl<'a, P> Sync for MatchIndices<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for Matches<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for RMatchIndices<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for RMatches<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for sp_std::str::RSplit<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for sp_std::str::RSplitN<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for RSplitTerminator<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for sp_std::str::Split<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for SplitInclusive<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for sp_std::str::SplitN<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, P> Sync for SplitTerminator<'a, P> where
    <P as Pattern<'a>>::Searcher: Sync, 

impl<'a, T> !Sync for sp_std::sync::mpsc::Iter<'a, T>

impl<'a, T> !Sync for TryIter<'a, T>

impl<'a, T> Sync for Difference<'a, T> where
    T: Sync, 

impl<'a, T> Sync for Intersection<'a, T> where
    T: Sync, 

impl<'a, T> Sync for sp_std::collections::btree_set::Iter<'a, T> where
    T: Sync, 

impl<'a, T> Sync for sp_std::collections::btree_set::Range<'a, T> where
    T: Sync, 

impl<'a, T> Sync for SymmetricDifference<'a, T> where
    T: Sync, 

impl<'a, T> Sync for Union<'a, T> where
    T: Sync, 

impl<'a, T> Sync for sp_std::collections::vec_deque::Iter<'a, T> where
    T: Sync, 

impl<'a, T> Sync for sp_std::result::Iter<'a, T> where
    T: Sync, 

impl<'a, T> Sync for sp_std::result::IterMut<'a, T> where
    T: Sync, 

impl<'a, T> Sync for Chunks<'a, T> where
    T: Sync, 

impl<'a, T> Sync for ChunksExact<'a, T> where
    T: Sync, 

impl<'a, T> Sync for ChunksExactMut<'a, T> where
    T: Sync, 

impl<'a, T> Sync for ChunksMut<'a, T> where
    T: Sync, 

impl<'a, T> Sync for RChunks<'a, T> where
    T: Sync, 

impl<'a, T> Sync for RChunksExact<'a, T> where
    T: Sync, 

impl<'a, T> Sync for RChunksExactMut<'a, T> where
    T: Sync, 

impl<'a, T> Sync for RChunksMut<'a, T> where
    T: Sync, 

impl<'a, T> Sync for Windows<'a, T> where
    T: Sync, 

impl<'a, T, F> Sync for sp_std::collections::btree_set::DrainFilter<'a, T, F> where
    F: Sync,
    T: Sync, 

impl<'a, T, F, A> Sync for sp_std::vec::DrainFilter<'a, T, F, A> where
    A: Sync,
    F: Sync,
    T: Sync, 

impl<'a, T, P> Sync for GroupBy<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for GroupByMut<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for sp_std::slice::RSplit<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for RSplitMut<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for sp_std::slice::RSplitN<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for RSplitNMut<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for sp_std::slice::Split<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for SplitMut<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for sp_std::slice::SplitN<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, P> Sync for SplitNMut<'a, T, P> where
    P: Sync,
    T: Sync, 

impl<'a, T, const N: usize> !Sync for ArrayWindows<'a, T, N>

impl<'a, T, const N: usize> Sync for ArrayChunks<'a, T, N> where
    T: Sync, 

impl<'a, T, const N: usize> Sync for ArrayChunksMut<'a, T, N> where
    T: Sync, 

impl<'b, T> !Sync for Ref<'b, T>

impl<'b, T> !Sync for RefMut<'b, T>

impl<A> Sync for Repeat<A> where
    A: Sync, 

impl<A, B> Sync for Chain<A, B> where
    A: Sync,
    B: Sync, 

impl<A, B> Sync for Zip<A, B> where
    A: Sync,
    B: Sync, 

impl<B, C> Sync for ControlFlow<B, C> where
    B: Sync,
    C: Sync, 

impl<F> Sync for FromFn<F> where
    F: Sync, 

impl<F> Sync for OnceWith<F> where
    F: Sync, 

impl<F> Sync for RepeatWith<F> where
    F: Sync, 

impl<H> Sync for BuildHasherDefault<H> where
    H: Sync, 

impl<I> Sync for Cloned<I> where
    I: Sync, 

impl<I> Sync for Copied<I> where
    I: Sync, 

impl<I> Sync for Cycle<I> where
    I: Sync, 

impl<I> Sync for Enumerate<I> where
    I: Sync, 

impl<I> Sync for Flatten<I> where
    I: Sync,
    <<I as Iterator>::Item as IntoIterator>::IntoIter: Sync, 

impl<I> Sync for Fuse<I> where
    I: Sync, 

impl<I> Sync for Intersperse<I> where
    I: Sync,
    <I as Iterator>::Item: Sync, 

impl<I> Sync for Peekable<I> where
    I: Sync,
    <I as Iterator>::Item: Sync, 

impl<I> Sync for Skip<I> where
    I: Sync, 

impl<I> Sync for StepBy<I> where
    I: Sync, 

impl<I> Sync for Take<I> where
    I: Sync, 

impl<I, F> Sync for FilterMap<I, F> where
    F: Sync,
    I: Sync, 

impl<I, F> Sync for Inspect<I, F> where
    F: Sync,
    I: Sync, 

impl<I, F> Sync for Map<I, F> where
    F: Sync,
    I: Sync, 

impl<I, G> Sync for IntersperseWith<I, G> where
    G: Sync,
    I: Sync,
    <I as Iterator>::Item: Sync, 

impl<I, P> Sync for Filter<I, P> where
    I: Sync,
    P: Sync, 

impl<I, P> Sync for MapWhile<I, P> where
    I: Sync,
    P: Sync, 

impl<I, P> Sync for SkipWhile<I, P> where
    I: Sync,
    P: Sync, 

impl<I, P> Sync for TakeWhile<I, P> where
    I: Sync,
    P: Sync, 

impl<I, St, F> Sync for Scan<I, St, F> where
    F: Sync,
    I: Sync,
    St: Sync, 

impl<I, U, F> Sync for FlatMap<I, U, F> where
    F: Sync,
    I: Sync,
    <U as IntoIterator>::IntoIter: Sync, 

impl<Idx> Sync for sp_std::ops::Range<Idx> where
    Idx: Sync, 

impl<Idx> Sync for RangeFrom<Idx> where
    Idx: Sync, 

impl<Idx> Sync for RangeInclusive<Idx> where
    Idx: Sync, 

impl<Idx> Sync for RangeTo<Idx> where
    Idx: Sync, 

impl<Idx> Sync for RangeToInclusive<Idx> where
    Idx: Sync, 

impl<K, V> Sync for BTreeMap<K, V> where
    K: Sync,
    V: Sync, 

impl<K, V> Sync for sp_std::collections::btree_map::IntoIter<K, V> where
    K: Sync,
    V: Sync, 

impl<K, V> Sync for IntoKeys<K, V> where
    K: Sync,
    V: Sync, 

impl<K, V> Sync for IntoValues<K, V> where
    K: Sync,
    V: Sync, 

impl<T> !Sync for sp_std::sync::mpsc::IntoIter<T>

impl<T> Sync for Bound<T> where
    T: Sync, 

impl<T> Sync for TryLockError<T> where
    T: Sync, 

impl<T> Sync for TrySendError<T> where
    T: Sync, 

impl<T> Sync for Reverse<T> where
    T: Sync, 

impl<T> Sync for BTreeSet<T> where
    T: Sync, 

impl<T> Sync for sp_std::collections::btree_set::IntoIter<T> where
    T: Sync, 

impl<T> Sync for sp_std::collections::vec_deque::IntoIter<T> where
    T: Sync, 

impl<T> Sync for VecDeque<T> where
    T: Sync, 

impl<T> Sync for sp_std::iter::Once<T> where
    T: Sync, 

impl<T> Sync for Rev<T> where
    T: Sync, 

impl<T> Sync for Discriminant<T>

impl<T> Sync for Wrapping<T> where
    T: Sync, 

impl<T> Sync for sp_std::result::IntoIter<T> where
    T: Sync, 

impl<T> Sync for SendError<T> where
    T: Sync, 

impl<T> Sync for SyncSender<T> where
    T: Send, 

impl<T> Sync for PoisonError<T> where
    T: Sync, 

impl<T> Sync for MaybeUninit<T> where
    T: Sync, 

impl<T, A> Sync for Vec<T, A> where
    A: Sync,
    T: Sync, 

impl<T, E> Sync for Result<T, E> where
    E: Sync,
    T: Sync, 

impl<T, F> Sync for Successors<T, F> where
    F: Sync,
    T: Sync, 

impl<T: ?Sized> Sync for ManuallyDrop<T> where
    T: Sync, 

impl<T: ?Sized> Sync for PhantomData<T> where
    T: Sync, 

impl<T: ?Sized, A> Sync for Box<T, A> where
    A: Sync,
    T: Sync, 

impl<Y, R> Sync for GeneratorState<Y, R> where
    R: Sync,
    Y: Sync, 

Loading content...